﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Persistence;

namespace doLogic
{
    public class DivisionManagers : PersistentList<DivisionManager>
    {
        public Division Division { get; protected set; }

        public static DivisionManagers For(int divisionID)
        {
            if (divisionID < 1)
                throw new ArgumentOutOfRangeException("DivisionID is required.");

            using (Connection cn = Database.Open())
            {
                DivisionManagers list = new DivisionManagers();
                list.Load("ForDivision", new { DivisionID = divisionID }, cn);

                list.Division = Division.Load(divisionID, cn);

                foreach (DivisionManager dm in list)
                {
                    dm.Division = list.Division;
                    dm.LoadManager(cn);
                }
                return list;
            }
        }

        public DivisionManagers Matching(string searchCriteria)
        {
            if (searchCriteria.IsEmpty())
                return this;

            searchCriteria = searchCriteria.ToLower();

            DivisionManagers list = new DivisionManagers();

            foreach (DivisionManager dm in this)
                if (dm.Manager.Equals(searchCriteria))
                    list.Add(dm);
            return list;
        }
    }

    [Persistable("Manager_Divisions", "ID", false)]
    public class DivisionManager : ManagerDivision
    {
    }
}
