﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Persistence;

namespace doLogic
{
    public class Locations : PersistentList<Location>
    {
        public static Locations For(Client client)
        {
            return new Locations(client.ClientCode);
        }

        public static Locations For(Client client, Division division)
        {
            return new Locations(client.ClientCode, division.DivID);
        }

        public Locations WhereNameContains(string search)
        {
            search = search.ToLower();

            Locations locations = new Locations();

            foreach (Location l in this)
                if (l.LocationName.ToLower().Contains(search))
                    locations.Add(l);
            return locations;
        }

		public Locations ForDivision(int divisionID)
		{
			Locations list = new Locations();
			foreach (Location item in this)
				if (item.DivID == divisionID)
					list.Add(item);
			return list;
		}

        internal Locations() { }

        private Locations(string clientCode, int pageNumber, int pageSize)
        {
            this.ClientCode = Convert.ToInt32(clientCode);
            this.PageNumber = pageNumber;
            this.PageSize = pageSize;
            this.Load("ForClientOrDivision");
        }

        internal Locations(string clientCode) : this(Value.ToInt32(clientCode), 0) { }
		internal Locations(int clientCode) : this(clientCode, 0) { }
		internal Locations(int clientCode, int divID)
        {
            this.ClientCode = clientCode;
            this.DivID = divID;
            this.Load("ForClientOrDivision");
        }

        public int ClientCode { get; set; }
        public int DivID { get; set; }
    }

    [Persistable("Locations", "LocID", false)]
    public class Location
    {
        public override string ToString()
        {
            if (Value.IsEmpty(LocationName))
                return base.ToString();
            else
                return LocationName;
        }

        private Division _division = null;
        public Division Division
        {
            get
            {
                if (Value.IsNotEmpty(this.DivID) && Value.IsEmpty(_division))
                    _division = Division.Load(this.DivID);
                return _division;
            }
        }

        public static Location Load(int locationID)
        {
            return new Location(locationID);
        }

        public static Location Load(int locationID, Connection cn)
        {
            return new Location(locationID, cn);
        }

        //constructors
        public Location() { }

        public Location(int locationID)
        {
            LocID = locationID;
            Database.Load(this);
        }

        private Location(int locationID, Connection cn)
        {
            LocID = locationID;
            Database.Load(this, cn);
        }

        public string DivisionName { get; set; }

        //persistent properties
        public int LocID { get; set; }
        public string LocationName { get; set; }
        public int DivID { get; set; }
        public int ClientCode { get; set; }
        public int EDNH { get; set; }
        public int CODNH { get; set; }
        public int EDRE { get; set; }
        public int CODRE { get; set; }
        public int EDDC { get; set; }
        public int CODDC { get; set; }
        public int EDCCEL { get; set; }
        public int CODCCEL { get; set; }
        public int EDCCM { get; set; }
        public int CODCCM { get; set; }
        public int EDCCIN { get; set; }
        public int CODCCIN { get; set; }

        public int Store() { return Database.Store(this); }
        public bool Delete() { return Database.Delete(this); }
    }
}
